/*
 *	NEWS-0.4.4 - RSS Feed Ticker - Copyright (C) Emmanuel Thomas-Maurin 2009-2010
 *	< manutm007@gmail.com >
 *
 * 	This program is free software: you can redistribute it and/or modify
 * 	it under the terms of the GNU General Public License as published by
 * 	the Free Software Foundation, either version 3 of the License, or
 * 	(at your option) any later version.
 *
 * 	This program is distributed in the hope that it will be useful,
 * 	but WITHOUT ANY WARRANTY; without even the implied warranty of
 * 	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * 	GNU General Public License for more details.
 *
 * 	You should have received a copy of the GNU General Public License
 * 	along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#include "news.h"

#ifdef G_OS_WIN32
extern FILE	*stdout_fp, *stderr_fp;
#endif

void display_time(const Params *prm)
{
	GrEnv			*env = get_gr_env();
	GdkDrawable		*pixmap2;
	PangoLayout		*p_layout;
	PangoFontDescription	*f_des;
	int			layout_width, layout_height;
	GdkGC			*gc2;
	GdkColor		shadow_color;
	guint16			shadow_color_alpha;
	float			shadow_k;
	char			tmp[8 + 1];
	time_t			time2;
	gint			clock_width;	/*v_diff*/
	gint			h_diff;

	if ((prm->clock == 'l' || prm->clock == 'r') && env->gc != NULL) {
		if ((p_layout = pango_layout_new(gtk_widget_get_pango_context(env->win))) == NULL)
			return;
		pango_layout_set_attributes(p_layout, NULL);
		pango_layout_set_single_paragraph_mode(p_layout, TRUE);

		f_des = pango_font_description_from_string((const char *)prm->clock_font_n_s);
		pango_layout_set_font_description(p_layout, f_des);
		pango_font_description_free(f_des);

		time2 = time(NULL);
		str_n_cpy(tmp, ctime(&time2) + 11, 8);
		pango_layout_set_text(p_layout, tmp, -1);

		pango_layout_context_changed(p_layout);
		pango_layout_get_pixel_size2(p_layout, &layout_width, &layout_height);

		clock_width = get_clock_width(prm);
		/*
		 * for some fonts like 'impact bold' which give strange things
		 * v_diff = clock_width - layout_width;
		 */
		gtk_widget_set_size_request(env->drwa_clock, clock_width, env->drwa_height);
		h_diff = (env->drwa_height - (gint)layout_height) / 2;

		if ((pixmap2 = gdk_pixmap_new(NULL, (gint)layout_width, (gint)layout_height, env->depth)) == NULL) {
			if (p_layout != NULL)
				g_object_unref(p_layout);
			return;
		}
		/*
		 * draw clock drawing area background
		 */
		if (!GDK_IS_DRAWABLE((env->drwa_clock)->window)) {
			if (p_layout != NULL)
				g_object_unref(p_layout);
			return;
		}
		if (env->gc == NULL || !GDK_IS_GC(env->gc)) {
			if (p_layout != NULL)
				g_object_unref(p_layout);
			return;
		}
		if (env->colormap == NULL || !GDK_IS_COLORMAP(env->colormap)) {
			if (p_layout != NULL)
				g_object_unref(p_layout);
			return;
		}

		gc2 = gdk_gc_new(pixmap2);
		if (!GDK_IS_GC(gc2)) {
			if (p_layout != NULL)
				g_object_unref(p_layout);
			return;
		}
		gdk_gc_copy(gc2, env->gc);
		gdk_gc_set_colormap(env->gc, env->colormap);
		gdk_gc_set_rgb_fg_color(gc2, &prm->clock_bg_color);
		pango_layout_context_changed(p_layout);

		gdk_draw_rectangle(GDK_DRAWABLE((env->drwa_clock)->window),
			gc2, TRUE, 0, 0, clock_width, env->drwa_height);

		gdk_gc_copy(gc2, env->gc);
		pango_layout_context_changed(p_layout);

		if (prm->shadow == 'y') {
			/*
			 * compute shadow color
			 */
			if (prm->shadow_fx < 0)
				shadow_k = 1.0;
			else if (prm->shadow_fx > 10)
				shadow_k = 0.0;
			else
				shadow_k = 1.0 - (float)prm->shadow_fx / 10.0;

			shadow_color.red = (guint16)((float)prm->clock_bg_color.red * shadow_k);
			shadow_color.green = (guint16)((float)prm->clock_bg_color.green * shadow_k);
			shadow_color.blue = (guint16)((float)prm->clock_bg_color.blue * shadow_k);
			shadow_color_alpha = prm->clock_bg_color_alpha;

			gdk_draw_layout_with_colors(pixmap2, gc2,
				0, 0, p_layout, NULL, &prm->clock_bg_color);
			gdk_draw_layout_with_colors(pixmap2, gc2,
				prm->shadow_offset_x, prm->shadow_offset_y,
				p_layout, &shadow_color, &prm->clock_bg_color);
			gdk_draw_layout_with_colors(pixmap2, gc2, 0, 0, p_layout,
				&prm->clock_fg_color, NULL);
		} else
			gdk_draw_layout_with_colors(pixmap2, gc2, 0, 0, p_layout,
				&prm->clock_fg_color, &prm->clock_bg_color);
		if (p_layout != NULL)
			g_object_unref(p_layout);

		gdk_draw_drawable(GDK_DRAWABLE((env->drwa_clock)->window),
			gc2, pixmap2, 0, 0, 0, h_diff, clock_width, (gint)layout_height);

		g_object_unref(pixmap2);
		g_object_unref(gc2);
	}
	return;
}

/* actually clock max width */
gint get_clock_width(const Params *prm)
{
	if (prm->clock == 'l' || prm->clock == 'r') {
		PangoLayout		*p_layout;
		PangoFontDescription	*f_des;
		int			layout_width, layout_height, layout_max_width = 0;
		char			tmp[8 + 1];
		int			i;

		p_layout = pango_layout_new(gtk_widget_get_pango_context(get_gr_env()->win));
		pango_layout_set_attributes(p_layout, NULL);
		pango_layout_set_single_paragraph_mode(p_layout, TRUE);

		f_des = pango_font_description_from_string((const char *)prm->clock_font_n_s);
		pango_layout_set_font_description(p_layout, f_des);
		pango_font_description_free(f_des);

		for (i = '0'; i <= '9'; i++) {
			snprintf(tmp, 8 + 1, "%c%c:%c%c:%c%c", i, i, i, i, i, i);
			pango_layout_set_text(p_layout, tmp, -1);
			pango_layout_context_changed(p_layout);
			pango_layout_get_pixel_size2(p_layout, &layout_width, &layout_height);
			if (layout_width > layout_max_width)
				layout_max_width = layout_width;
		}
		if (p_layout != NULL)
			g_object_unref(p_layout);
		return (gint)layout_max_width;
	} else
		return 0;
}
