/*
 *	NEWS-0.4.4 - RSS Feed Ticker - Copyright (C) Emmanuel Thomas-Maurin 2009-2010
 *	< manutm007@gmail.com >
 *
 * 	This program is free software: you can redistribute it and/or modify
 * 	it under the terms of the GNU General Public License as published by
 * 	the Free Software Foundation, either version 3 of the License, or
 * 	(at your option) any later version.
 *
 * 	This program is distributed in the hope that it will be useful,
 * 	but WITHOUT ANY WARRANTY; without even the implied warranty of
 * 	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * 	GNU General Public License for more details.
 *
 * 	You should have received a copy of the GNU General Public License
 * 	along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#include "news.h"

#if USE_GUI

extern const char	*help_str0[];
extern const char	*help_str1[];
extern const char	*license_str1[];
extern const char	*license_str2;

static gint esc_key_pressed(GtkWidget *dialog, GdkEventAny *event, gpointer unused)
{
	unused = unused;
	if (event->type == GDK_KEY_PRESS && ((GdkEventKey *)event)->keyval == GDK_Escape) {
		gtk_dialog_response(GTK_DIALOG(dialog), GTK_RESPONSE_CANCEL);
		return TRUE;
	} else
		return FALSE;
}

/*
 * pick new file dialog win
 */
void open_new_txt_file(Resource *resrc, int *rss_ttl)
{
	GrEnv		*env = get_gr_env();
	GtkWidget	*dialog;
	char		*file_name2, tmp[TMPSTR_SIZE + 1];
	int		resource_flag;
	Resource	resrc_bak;

	gtk_window_set_keep_above(GTK_WINDOW(get_gr_env()->win), FALSE);

	dialog = gtk_file_chooser_dialog_new("File Picker", GTK_WINDOW(env->win),
			GTK_FILE_CHOOSER_ACTION_OPEN,
			GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL,
			GTK_STOCK_OK, GTK_RESPONSE_OK,
			NULL);
#ifndef G_OS_WIN32
	gtk_window_set_icon_from_file(GTK_WINDOW(dialog), NEWS_ICON, NULL);
#else
	str_n_cpy(tmp, get_progfiles_dir(), TMPSTR_SIZE - 64);
	str_n_cat(tmp, "\\"NEWS_ICON, 64);
	gtk_window_set_icon_from_file(GTK_WINDOW(dialog), tmp, NULL);
#endif
	gtk_window_set_position(GTK_WINDOW(dialog), GTK_WIN_POS_CENTER);

	g_signal_connect(G_OBJECT(dialog), "key-press-event", G_CALLBACK(esc_key_pressed), "key-press-event");
	gtk_widget_show_all(dialog);
	/*
	 * backup last valid opened resource (if any)
	 */
	str_n_cpy(resrc_bak.id, resrc->id, FILENAME_MAXLEN);

	while (gtk_dialog_run(GTK_DIALOG(dialog)) == GTK_RESPONSE_OK) {
		file_name2 = gtk_file_chooser_get_filename(GTK_FILE_CHOOSER(dialog));
		str_n_cpy(resrc->id, file_name2, FILENAME_MAXLEN);
		g_free(file_name2);

		env->suspend_rq = TRUE;
		if ((resource_flag = load_resource(resrc, rss_ttl)) == RESOURCE_OK) {
			env->reload_rq = TRUE;
			env->suspend_rq = FALSE;
			break;
		} else {
			snprintf(tmp, TMPSTR_SIZE + 1, "load_resource() error %d", resource_flag);
			info_win(tmp);
			str_n_cpy(resrc->id, resrc_bak.id, FILENAME_MAXLEN);
			load_resource(resrc, rss_ttl);
			env->suspend_rq = FALSE;
		}
	}
	gtk_widget_destroy(dialog);
	check_main_win_always_on_top();
	return;
}

static void get_layout_dims(GtkWidget *view, int *layout_width, int *layout_height, char *str)
{
	PangoLayout	*p_layout;

	p_layout = pango_layout_new(gtk_widget_get_pango_context(view));
	pango_layout_set_text(p_layout, str, -1);
	pango_layout_get_pixel_size2(p_layout, layout_width, layout_height);
	if (p_layout != NULL)
		g_object_unref(p_layout);
	return;
}

/*static gint get_str_width_in_context(GtkWidget *win, char *str)
{
	PangoLayout	*p_layout;
	gint		layout_width, layout_height;

	p_layout = pango_layout_new(gtk_widget_get_pango_context(win));
	pango_layout_set_attributes(p_layout, NULL);
	pango_layout_set_single_paragraph_mode(p_layout, TRUE);

	pango_layout_set_text(p_layout, str, -1);
	pango_layout_get_pixel_size(p_layout, &layout_width, &layout_height);

	g_object_unref(p_layout);

	return layout_width;
}

static char *format_help_txt(GtkWidget *win, char *txt)
{
	char		tmp1[TMPSTR_SIZE + 1], tmp2[TMPSTR_SIZE + 1], tmp3[TMPSTR_SIZE + 1];
	int		i, j, tmp1_width;

	*//* a little bit messy *//*
	for (i = 0; help_str1[i] != NULL; i++) {
		*//*
		 * spacing adjustement is needed here because we don't use a
		 * width-fixed font anymore like in text mode - the following
		 * code works fine but is a bit confusing
		 *//*
		str_n_cpy(tmp1, (char *)help_str1[i], 128);
		j = strlen(tmp1);
		while (strncmp(tmp1 + j--, "  ", 2) != 0);
		str_n_cpy(tmp2, tmp1 + j + 3, 64);
		while (strncmp(tmp1 + j--, "  ", 2) == 0);
		tmp1[j + 2] = '\0';
		tmp1_width = get_str_width_in_context(win, tmp1);
		str_n_cpy(tmp3, " ", 1);
		while (tmp1_width + get_str_width_in_context(win, tmp3) <= 250)
			str_n_cat(tmp3, " ", 1);
		str_n_cat(tmp1, tmp3, 64);
		str_n_cat(tmp1, tmp2, 64);
		txt = l_str_cat(txt, tmp1);
	}
	return txt;
}*/

void help_win()
{
#ifndef G_OS_WIN32
# define FONT			"DejaVu Sans Mono 8" /* 8 9 10 ok */
#else
# define FONT			"Courier New 8"
#endif
#define HLPWIN_FG_COLOR		"#000000ff"	/* black */
#define HLPWIN_BG_COLOR		"#ffffffff"	/* white */

	GtkWidget		*dialog, *sc_win, *view;
	GtkTextBuffer		*buf;
	PangoFontDescription	*font_des;
	GdkColor		fg_color, bg_color;
	guint16			fgc_alpha, bgc_alpha;
	char			*txt;
#ifdef G_OS_WIN32
	char			tmp[TMPSTR_SIZE + 1];
#endif
	int			width, height;
	int			i;

	gtk_window_set_keep_above(GTK_WINDOW(get_gr_env()->win), FALSE);

	dialog = gtk_dialog_new_with_buttons("News Quick Help", GTK_WINDOW(get_gr_env()->win),
		GTK_DIALOG_MODAL | GTK_DIALOG_DESTROY_WITH_PARENT,
		GTK_STOCK_CLOSE, GTK_RESPONSE_CLOSE, NULL);
#ifndef G_OS_WIN32
	gtk_window_set_icon_from_file(GTK_WINDOW(dialog), NEWS_ICON, NULL);
#else
	str_n_cpy(tmp, get_progfiles_dir(), TMPSTR_SIZE - 64);
	str_n_cat(tmp, "\\"NEWS_ICON, 64);
	gtk_window_set_icon_from_file(GTK_WINDOW(dialog), tmp, NULL);
#endif
	gtk_window_set_position(GTK_WINDOW(dialog), GTK_WIN_POS_CENTER);

	sc_win = gtk_scrolled_window_new(NULL, NULL);
	gtk_scrolled_window_set_policy(GTK_SCROLLED_WINDOW(sc_win), GTK_POLICY_AUTOMATIC,
		GTK_POLICY_AUTOMATIC);
	gtk_container_add(GTK_CONTAINER(GTK_DIALOG(dialog)->vbox), sc_win);
	gtk_container_set_border_width(GTK_CONTAINER(sc_win), 5);

	txt = l_str_new(NULL);
	buf = gtk_text_buffer_new(NULL);

	view = gtk_text_view_new_with_buffer(buf);
	gtk_text_view_set_editable(GTK_TEXT_VIEW(view), FALSE);
	gtk_text_view_set_cursor_visible(GTK_TEXT_VIEW(view), FALSE);
	font_des = pango_font_description_from_string(FONT);
	gtk_widget_modify_font(view, font_des);

	get_gdk_color_from_hexstr(HLPWIN_FG_COLOR, &fg_color, &fgc_alpha);
	get_gdk_color_from_hexstr(HLPWIN_BG_COLOR, &bg_color, &bgc_alpha);
	/*
	 * not using gtk_widget_modify_fg/bg() but gtk_widget_modify_text/base()
	 * because it's specifically for editable text
	 */
	if (FALSE) {	/* one possible extra setting = 'same colors as ticker's' = 'yes' */
		gtk_widget_modify_text(view, GTK_STATE_NORMAL, &(get_params()->fg_color));
		gtk_widget_modify_base(view, GTK_STATE_NORMAL, &(get_params()->bg_color));
	} else if (TRUE) { /* widget-level-defined */
		gtk_widget_modify_text(view, GTK_STATE_NORMAL, &fg_color);
		gtk_widget_modify_base(view, GTK_STATE_NORMAL, &bg_color);
	} else if (FALSE) { /* same as label inside window */
		gtk_widget_modify_text(view, GTK_STATE_NORMAL,
			(const GdkColor *)&(gtk_widget_get_style(get_gr_env()->win)->fg[GTK_STATE_NORMAL]));
		gtk_widget_modify_base(view, GTK_STATE_NORMAL,
			(const GdkColor *)&(gtk_widget_get_style(get_gr_env()->win)->bg[GTK_STATE_NORMAL]));
	}

	gtk_text_view_set_left_margin(GTK_TEXT_VIEW(view), 15);
	gtk_text_view_set_right_margin(GTK_TEXT_VIEW(view), 15);
	gtk_container_add(GTK_CONTAINER(sc_win), view);

	txt = l_str_cat(txt, "\n");
	for (i = 0; help_str0[i] != NULL; i++)
		txt = l_str_cat(txt, help_str0[i]);
	for (i = 0; help_str1[i] != NULL; i++)
		txt = l_str_cat(txt, help_str1[i]);

	gtk_text_buffer_set_text(buf, txt, -1);
	get_layout_dims(view, &width, &height, txt);
	l_str_free(txt);
	if (width < 400)
		width = 400;
	else if (width > 800)
		width = 800;
	if (height > 450)
		height = 450;
	gtk_widget_set_size_request(sc_win, width + 30 + 15 + 15, height);

	gtk_widget_show_all(dialog);
	gtk_dialog_run(GTK_DIALOG(dialog));

	gtk_widget_destroy(dialog);
	check_main_win_always_on_top();
	return;
}

static gint license_win()
{
	GtkWidget	*dialog, *label;
	char		*txt;
#ifdef G_OS_WIN32
	char		tmp[TMPSTR_SIZE + 1];
#endif
	int		i;

	gtk_window_set_keep_above(GTK_WINDOW(get_gr_env()->win), FALSE);

	dialog = gtk_dialog_new_with_buttons("News License", GTK_WINDOW(get_gr_env()->win),
		GTK_DIALOG_MODAL | GTK_DIALOG_DESTROY_WITH_PARENT,
		GTK_STOCK_CLOSE, GTK_RESPONSE_CLOSE, NULL);
#ifndef G_OS_WIN32
	gtk_window_set_icon_from_file(GTK_WINDOW(dialog), NEWS_ICON, NULL);
#else
	str_n_cpy(tmp, get_progfiles_dir(), TMPSTR_SIZE - 64);
	str_n_cat(tmp, "\\"NEWS_ICON, 64);
	gtk_window_set_icon_from_file(GTK_WINDOW(dialog), tmp, NULL);
#endif
	gtk_container_set_border_width(GTK_CONTAINER(dialog), 15);
	gtk_window_set_position(GTK_WINDOW(dialog), GTK_WIN_POS_CENTER);


#ifndef G_OS_WIN32
	txt = l_str_new("<small>");
#else
	txt = l_str_new(NULL);
#endif
	for (i = 0; license_str1[i] != NULL; i++)
		txt = l_str_cat(txt, license_str1[i]);
#ifndef G_OS_WIN32
	txt = l_str_cat(txt, "<a href=\"http://www.gnu.org/licenses/\">http://www.gnu.org/licenses/</a>");
	txt = l_str_cat(txt, "</small>");
#else
	txt = l_str_cat(txt, "<http://www.gnu.org/licenses/>");
#endif
	txt = l_str_cat(txt, "\n");

	label = gtk_label_new(txt);
	gtk_label_set_use_markup(GTK_LABEL(label), TRUE);
	gtk_container_add(GTK_CONTAINER(GTK_DIALOG(dialog)->vbox), label);
	l_str_free(txt);

	gtk_widget_show_all(dialog);
	gtk_dialog_run(GTK_DIALOG(dialog));

	gtk_widget_destroy(dialog);
	check_main_win_always_on_top();
	return TRUE;
}

void about_win()
{
	GtkWidget	*dialog, *table, *image, *label[4];
	GtkWidget	*license_but, *close_but;
#ifdef G_OS_WIN32
	char		tmp[TMPSTR_SIZE + 1];
#endif
	gint		response;

	gtk_window_set_keep_above(GTK_WINDOW(get_gr_env()->win), FALSE);

	dialog = gtk_dialog_new_with_buttons("About News", GTK_WINDOW(get_gr_env()->win),
		 GTK_DIALOG_MODAL | GTK_DIALOG_DESTROY_WITH_PARENT,
		 NULL);

	license_but = gtk_dialog_add_button(GTK_DIALOG(dialog), "License", GTK_RESPONSE_NONE);
	close_but = gtk_dialog_add_button(GTK_DIALOG(dialog), GTK_STOCK_CLOSE, GTK_RESPONSE_CLOSE);

	g_signal_connect(G_OBJECT(license_but), "clicked", G_CALLBACK(license_win), NULL);
	g_signal_connect(G_OBJECT(dialog), "key-press-event", G_CALLBACK(esc_key_pressed), "key-press-event");

#ifndef G_OS_WIN32
	gtk_window_set_icon_from_file(GTK_WINDOW(dialog), NEWS_ICON, NULL);
#else
	str_n_cpy(tmp, get_progfiles_dir(), TMPSTR_SIZE - 64);
	str_n_cat(tmp, "\\"NEWS_ICON, 64);
	gtk_window_set_icon_from_file(GTK_WINDOW(dialog), tmp, NULL);
#endif
	gtk_window_set_position(GTK_WINDOW(dialog), GTK_WIN_POS_CENTER);

	table = gtk_table_new(7, 1, FALSE);
	gtk_table_set_row_spacings(GTK_TABLE(table), 5);
	gtk_container_set_border_width(GTK_CONTAINER(table), 15);
	gtk_container_add(GTK_CONTAINER(GTK_DIALOG(dialog)->vbox), table);

#ifndef G_OS_WIN32
	image = gtk_image_new_from_file(NEWS_LOGO);
#else
	str_n_cpy(tmp, get_progfiles_dir(), TMPSTR_SIZE - 64);
	str_n_cat(tmp, "\\"NEWS_LOGO, 64);
	image = gtk_image_new_from_file(tmp);
#endif
	gtk_table_attach_defaults(GTK_TABLE(table), image, 0, 1, 0, 1);

	label[0] = gtk_label_new("\n<b>NEWS version "APP_VERSION_NUMBER" - RSS Feed Ticker</b>");
	gtk_label_set_use_markup(GTK_LABEL(label[0]), TRUE);
	gtk_table_attach_defaults(GTK_TABLE(table), label[0], 0, 1, 1, 2);
#ifndef G_OS_WIN32
	label[1] = gtk_label_new("<small>Copyright (C) Emmanuel Thomas-Maurin 2009-2010</small>");
#else
	label[1] = gtk_label_new("Copyright (C) Emmanuel Thomas-Maurin 2009-2010");
#endif
	gtk_label_set_use_markup(GTK_LABEL(label[1]), TRUE);
	gtk_table_attach_defaults(GTK_TABLE(table), label[1], 0, 1, 2, 3);
#ifndef G_OS_WIN32
	label[2] = gtk_label_new("<a href=\"mailto:"APP_SUPPORT_EMAIL"\">"APP_SUPPORT_EMAIL"</a>");
#else
	label[2] = gtk_label_new(APP_SUPPORT_EMAIL);
#endif
	gtk_label_set_use_markup(GTK_LABEL(label[2]), TRUE);
	gtk_table_attach_defaults(GTK_TABLE(table), label[2], 0, 1, 3, 4);
#ifndef G_OS_WIN32
	label[3] = gtk_label_new("<a href=\""APP_WEBSITE"\">"APP_WEBSITE"</a>");
#else
	label[3] = gtk_label_new(APP_WEBSITE);
#endif
	gtk_label_set_use_markup(GTK_LABEL(label[3]), TRUE);
	gtk_table_attach_defaults(GTK_TABLE(table), label[3], 0, 1, 4, 5);

	gtk_widget_show_all(dialog);
	while ((response = gtk_dialog_run(GTK_DIALOG(dialog))) != GTK_RESPONSE_CLOSE \
		&& response != GTK_RESPONSE_CANCEL);

	gtk_widget_destroy(dialog);
	check_main_win_always_on_top();
	return;
}

/*
 * info, warning or error
 * === should format txt in case it's a long string ===
 */
void info_win(const char *txt)
{
	GtkWidget	*dialog, *table, *image, *label[2];
#ifdef G_OS_WIN32
	char		tmp[TMPSTR_SIZE + 1];
#endif

	gtk_window_set_keep_above(GTK_WINDOW(get_gr_env()->win), FALSE);

	dialog = gtk_dialog_new_with_buttons("News", GTK_WINDOW(get_gr_env()->win),
		 GTK_DIALOG_MODAL | GTK_DIALOG_DESTROY_WITH_PARENT,
		 GTK_STOCK_OK, GTK_RESPONSE_OK, NULL);
#ifndef G_OS_WIN32
	gtk_window_set_icon_from_file(GTK_WINDOW(dialog), NEWS_ICON, NULL);
#else
	str_n_cpy(tmp, get_progfiles_dir(), TMPSTR_SIZE - 64);
	str_n_cat(tmp, "\\"NEWS_ICON, 64);
	gtk_window_set_icon_from_file(GTK_WINDOW(dialog), tmp, NULL);
#endif
	gtk_window_set_position(GTK_WINDOW(dialog), GTK_WIN_POS_CENTER);

	table = gtk_table_new(1, 3, FALSE);
	gtk_table_set_col_spacings(GTK_TABLE(table), 10);
	gtk_container_set_border_width(GTK_CONTAINER(table), 0);
	gtk_container_add(GTK_CONTAINER(GTK_DIALOG(dialog)->vbox), table);

	image = gtk_image_new_from_stock(GTK_STOCK_DIALOG_WARNING,
							GTK_ICON_SIZE_DIALOG);
	gtk_table_attach_defaults(GTK_TABLE(table), image, 0, 1, 0, 1);
	label[0] = gtk_label_new(txt);
	gtk_label_set_use_markup(GTK_LABEL(label[0]), TRUE);
	gtk_table_attach_defaults(GTK_TABLE(table), label[0], 1, 2, 0, 1);
	label[1] = gtk_label_new("");
	gtk_table_attach_defaults(GTK_TABLE(table), label[1], 2, 3, 0, 1);

	gtk_widget_show_all(dialog);
	gtk_dialog_run(GTK_DIALOG(dialog));

	gtk_widget_destroy(dialog);
	check_main_win_always_on_top();
	return;
}

/* delay in ms */
void info_win_wait(const char *txt, int delay)
{
	GtkWidget		*win;
	GtkWidget		*table, *image, *label[2];
#ifdef G_OS_WIN32
	char			tmp[TMPSTR_SIZE + 1];
#endif

	gtk_window_set_keep_above(GTK_WINDOW(get_gr_env()->win), FALSE);

	win = gtk_window_new(GTK_WINDOW_POPUP);
#ifndef G_OS_WIN32
	gtk_window_set_icon_from_file(GTK_WINDOW(win), NEWS_ICON, NULL);
#else
	str_n_cpy(tmp, get_progfiles_dir(), TMPSTR_SIZE - 64);
	str_n_cat(tmp, "\\"NEWS_ICON, 64);
	gtk_window_set_icon_from_file(GTK_WINDOW(win), tmp, NULL);
#endif
	gtk_window_set_position(GTK_WINDOW(win), GTK_WIN_POS_CENTER);

	table = gtk_table_new(1, 3, FALSE);
	gtk_table_set_col_spacings(GTK_TABLE(table), 10);
	gtk_container_set_border_width(GTK_CONTAINER(table), 0);
	gtk_container_add(GTK_CONTAINER(GTK_WINDOW(win)), table);

	image = gtk_image_new_from_stock(GTK_STOCK_DIALOG_WARNING,
						GTK_ICON_SIZE_DIALOG);
	gtk_table_attach_defaults(GTK_TABLE(table), image, 0, 1, 0, 1);
	label[0] = gtk_label_new(txt);
	gtk_label_set_use_markup(GTK_LABEL(label[0]), TRUE);
	gtk_table_attach_defaults(GTK_TABLE(table), label[0], 1, 2, 0, 1);
	label[1] = gtk_label_new("");
	gtk_table_attach_defaults(GTK_TABLE(table), label[1], 2, 3, 0, 1);

	gtk_widget_show_all(win);
	gdk_window_invalidate_rect(gtk_widget_get_window(win), NULL, TRUE);
	gdk_window_process_updates(gtk_widget_get_window(win), TRUE);

#ifndef G_OS_WIN32
	sleep(delay / 1000);
#else
	Sleep(delay);
#endif
	gtk_widget_destroy(win);
	check_main_win_always_on_top();
	return;
}

int question_win(const char *txt)
{
	GtkWidget	*dialog, *table, *image, *label[2];
	gint		response;
#ifdef G_OS_WIN32
	char		tmp[TMPSTR_SIZE + 1];
#endif

	gtk_window_set_keep_above(GTK_WINDOW(get_gr_env()->win), FALSE);

	dialog = gtk_dialog_new_with_buttons("News", GTK_WINDOW(get_gr_env()->win),
		 GTK_DIALOG_MODAL | GTK_DIALOG_DESTROY_WITH_PARENT,
		 GTK_STOCK_NO, GTK_RESPONSE_NO,
		 GTK_STOCK_YES, GTK_RESPONSE_YES,
		 NULL);
#ifndef G_OS_WIN32
	gtk_window_set_icon_from_file(GTK_WINDOW(dialog), NEWS_ICON, NULL);
#else
	str_n_cpy(tmp, get_progfiles_dir(), TMPSTR_SIZE - 64);
	str_n_cat(tmp, "\\"NEWS_ICON, 64);
	gtk_window_set_icon_from_file(GTK_WINDOW(dialog), tmp, NULL);
#endif
	gtk_window_set_position(GTK_WINDOW(dialog), GTK_WIN_POS_CENTER);

	gtk_dialog_set_default_response(GTK_DIALOG(dialog), GTK_RESPONSE_NO);

	table = gtk_table_new(1, 3, FALSE);
	gtk_table_set_col_spacings(GTK_TABLE(table), 10);
	gtk_container_set_border_width(GTK_CONTAINER(table), 0);
	gtk_container_add(GTK_CONTAINER(GTK_DIALOG(dialog)->vbox), table);

	image = gtk_image_new_from_stock(GTK_STOCK_DIALOG_QUESTION,
							GTK_ICON_SIZE_DIALOG);
	gtk_table_attach_defaults(GTK_TABLE(table), image, 0, 1, 0, 1);
	label[0] = gtk_label_new(txt);
	gtk_table_attach_defaults(GTK_TABLE(table), label[0], 1, 2, 0, 1);
	label[1] = gtk_label_new("");
	gtk_table_attach_defaults(GTK_TABLE(table), label[1], 2, 3, 0, 1);

	gtk_widget_show_all(dialog);
	response = gtk_dialog_run(GTK_DIALOG(dialog));

	gtk_widget_destroy(dialog);
	check_main_win_always_on_top();
	return ((response == GTK_RESPONSE_YES) ? YES : NO);
}

int question_win_always_on_top(const char *txt)
{
	GtkWidget	*dialog, *table, *image, *label[2];
	gint		response;
#ifdef G_OS_WIN32
	char		tmp[TMPSTR_SIZE + 1];
#endif

	gtk_window_set_keep_above(GTK_WINDOW(get_gr_env()->win), FALSE);

	dialog = gtk_dialog_new_with_buttons("News", GTK_WINDOW(get_gr_env()->win),
		 GTK_DIALOG_MODAL | GTK_DIALOG_DESTROY_WITH_PARENT,
		 GTK_STOCK_NO, GTK_RESPONSE_NO,
		 GTK_STOCK_YES, GTK_RESPONSE_YES,
		 NULL);
#ifndef G_OS_WIN32
	gtk_window_set_icon_from_file(GTK_WINDOW(dialog), NEWS_ICON, NULL);
#else
	str_n_cpy(tmp, get_progfiles_dir(), TMPSTR_SIZE - 64);
	str_n_cat(tmp, "\\"NEWS_ICON, 64);
	gtk_window_set_icon_from_file(GTK_WINDOW(dialog), tmp, NULL);
#endif
	gtk_window_set_position(GTK_WINDOW(dialog), GTK_WIN_POS_CENTER);
	gtk_window_set_keep_above(GTK_WINDOW(dialog), TRUE);

	gtk_dialog_set_default_response(GTK_DIALOG(dialog), GTK_RESPONSE_NO);

	table = gtk_table_new(1, 3, FALSE);
	gtk_table_set_col_spacings(GTK_TABLE(table), 10);
	gtk_container_set_border_width(GTK_CONTAINER(table), 0);
	gtk_container_add(GTK_CONTAINER(GTK_DIALOG(dialog)->vbox), table);

	image = gtk_image_new_from_stock(GTK_STOCK_DIALOG_QUESTION,
							GTK_ICON_SIZE_DIALOG);
	gtk_table_attach_defaults(GTK_TABLE(table), image, 0, 1, 0, 1);
	label[0] = gtk_label_new(txt);
	gtk_table_attach_defaults(GTK_TABLE(table), label[0], 1, 2, 0, 1);
	label[1] = gtk_label_new("");
	gtk_table_attach_defaults(GTK_TABLE(table), label[1], 2, 3, 0, 1);

	gtk_widget_show_all(dialog);
	response = gtk_dialog_run(GTK_DIALOG(dialog));

	gtk_widget_destroy(dialog);
	check_main_win_always_on_top();
	return ((response == GTK_RESPONSE_YES) ? YES : NO);
}
#endif
